; ::::::::::::::::::
; DE-LIGHTING TOOL
; ::::::::::::::::::


(define (deos-delighting inImg inLayer UI_KeepLayer UI_Info UI_Slider)
	(let*
		(	
			; VARIABLES
			(blurLayer 0)
			(blurLayer2 0)
			(editingLayer 0)
			(colorsLayer 0)
			(colorsLayer2 0)
			(editingLayer (car (gimp-drawable-get-name inLayer)))
			(imgWidth 0)
			(imgHeight 0)
			(imgSld 0)
		)
		
		; CHECK TEXTURE SIEZE ::::::::::::::::::
		( set! imgWidth  (car (gimp-image-width  inImg) ) )
		( set! imgHeight (car (gimp-image-height inImg) ) )
		
		; START PROCESS ::::::::::::::::::
		(gimp-context-push)
		(gimp-image-undo-group-start inImg)


		; DE-LIGHTING ::::::::::::::::::
		(if ( or (= imgWidth 512) (= imgWidth 1024) (= imgWidth 2048) (= imgWidth 4096) )
		
			; TEXTURA CORRECTA
			(begin	

				; Keep Layer
				(if (= UI_KeepLayer TRUE)
					(begin
						(set! editingLayer (car (gimp-layer-copy inLayer FALSE)))
						(gimp-image-add-layer inImg editingLayer -1)
						(gimp-drawable-set-name editingLayer "working")
					)
					;else
					(set! editingLayer inLayer)  
				)   

				; Select layer to work
				(gimp-image-set-active-layer inImg editingLayer)
			
				;Duplicate on top and blur
				(set! blurLayer (car (gimp-layer-copy editingLayer FALSE)))
				(gimp-image-add-layer inImg blurLayer -1)
				(gimp-drawable-set-name blurLayer "blur")

				; PROCESS 
				(if (= imgWidth 4096)
					
					(begin
						( set! imgSld (* UI_Slider 500 ) )
						(plug-in-gauss RUN-NONINTERACTIVE inImg blurLayer imgSld imgSld FALSE)
						(plug-in-gauss RUN-NONINTERACTIVE inImg blurLayer imgSld imgSld FALSE)
						; (gimp-message (string-append "4K Aplicado"))
					)
				)
					(if (= imgWidth 2048)
						(begin
							( set! imgSld (* UI_Slider 250 ) )
							(plug-in-gauss RUN-NONINTERACTIVE inImg blurLayer imgSld imgSld FALSE)	
							(plug-in-gauss RUN-NONINTERACTIVE inImg blurLayer imgSld imgSld FALSE)	
							; (gimp-message (string-append "2k Aplicado"))
						)
					
					)
						(if (= imgWidth 1024)
							(begin
								( set! imgSld (* UI_Slider 125 ) )
								(plug-in-gauss RUN-NONINTERACTIVE inImg blurLayer imgSld imgSld FALSE)	
								(plug-in-gauss RUN-NONINTERACTIVE inImg blurLayer imgSld imgSld FALSE)	
								; (gimp-message (string-append "1k Aplicado"))
							)
						)
							(if (= imgWidth 512)
								(begin
									( set! imgSld (* UI_Slider 62.5 ) )
									(plug-in-gauss RUN-NONINTERACTIVE inImg blurLayer imgSld imgSld FALSE)	
									(plug-in-gauss RUN-NONINTERACTIVE inImg blurLayer imgSld imgSld FALSE)	
									; (gimp-message (string-append "512k Aplicado"))
								)
							)

				(begin
					
					;Move on top layer
					(gimp-image-set-active-layer inImg blurLayer)  				
					
					;Get  average colour from img
					(set! colorsLayer (car (gimp-layer-copy inLayer FALSE)))	
					(gimp-image-add-layer inImg colorsLayer -1)
					(gimp-drawable-set-name colorsLayer "colors")

					(gimp-context-set-foreground (list 
						(car (gimp-histogram colorsLayer HISTOGRAM-RED 0 255)) 
						(car (gimp-histogram colorsLayer HISTOGRAM-GREEN 0 255)) 
						(car (gimp-histogram colorsLayer HISTOGRAM-BLUE 0 255)))
					)

					(gimp-drawable-fill colorsLayer FOREGROUND-FILL)
					(gimp-image-set-active-layer inImg colorsLayer)

					;Copy the Colour (Solid)
					(set! colorsLayer2 (car (gimp-layer-copy colorsLayer FALSE)))
					(gimp-image-add-layer inImg colorsLayer2 -1)		
					(gimp-layer-set-mode colorsLayer LAYER-MODE-SUBTRACT)
					(gimp-image-set-active-layer inImg colorsLayer2)

					;Copy Blurred Layer
					(set! blurLayer2 (car (gimp-layer-copy blurLayer FALSE)))
					(gimp-image-add-layer inImg blurLayer2 -1)		
					(gimp-layer-set-mode blurLayer2 LAYER-MODE-SUBTRACT)
					
					(set! blurLayer (car (gimp-image-merge-down inImg colorsLayer 0)))
					(set! blurLayer2 (car (gimp-image-merge-down inImg blurLayer2 0)))
					
					(gimp-layer-set-mode blurLayer LAYER-MODE-SUBTRACT)
					(gimp-layer-set-mode blurLayer2 LAYER-MODE-ADDITION)
					
					(set! editingLayer (car (gimp-image-merge-down inImg blurLayer 0)))
					(set! editingLayer (car (gimp-image-merge-down inImg blurLayer2 0)))
				)
			)

			; TEXTURA INCORRECTA
			(begin
				( gimp-message (string-append "This texture should be square in: 512, 1k, 2k or 4k. \n\n The current texture size is: " (number->string imgWidth) " x " (number->string imgHeight) "px" ) )
			)
		)

		; : : : : : : : : : : : : : : : : : 


		; EDNS PROGRESS ::::::::::::::::::
		(gimp-progress-end)
		(gimp-image-undo-group-end inImg)
		(gimp-displays-flush)
		(gimp-context-pop)

	)

)

; UI OPTIONS
(script-fu-register "deos-delighting"
		"De-Lighting"
		"Equilize lighting texture filter"
		"Deo"
		"Deo"
		"Feb, 2024"
		"RGB* GRAY*"
		SF-IMAGE			"image"														0
		SF-DRAWABLE		"drawable"												0
		SF-TOGGLE			"Keep Original Layer?"						TRUE
		SF-STRING			"Image should be only 'Square'" 	"| 512 | 1024 | 2048 | 4096 |"
		SF-ADJUSTMENT	"Balance of Lighting"							'(0.5 0.05 1 0.05 0.1 2 0)    ;Value, Min, Max, Step Min (Scroll), Step Max (Ctrl+Scroll), Digits, Silder Bool.
)
(script-fu-menu-register 	"deos-delighting"
													"<Image>/Filters/Deo's Tools")