; ::::::::::::::::::
; NORMAL MAP TOOL
; ::::::::::::::::::

(define (deos-MasterNormalMap inImg inLayer UI_KeepLayer UI_ApplyBlur UI_BlurPixels UI_Blending)
	(let*
		(	
			; VARIABLES
			(originLayer 0)
			(BackColor '(128 128 255))
			(fineDetailLayer 0)
			(mediumDetailLayer 0)
			(largeDetailLayer 0)
		)
		
		; START PROCESS ::::::::::::::::::
		(gimp-context-push)
		(gimp-image-undo-group-start inImg)
		

		; NORMAL MAPS ::::::::::::::::::

			; Background
			(set! originLayer (car (gimp-layer-copy inLayer FALSE)))
			(gimp-image-add-layer inImg originLayer -1)
			(gimp-drawable-set-name originLayer "BackGround")
			(gimp-context-set-foreground BackColor)
			(gimp-edit-fill originLayer FOREGROUND-FILL)


			; FINE NORMALMAP
			(set! fineDetailLayer (car (gimp-layer-copy inLayer FALSE)))
			(gimp-image-add-layer inImg fineDetailLayer -2)
			
			
			(if (= UI_ApplyBlur TRUE)
				(begin 
					(gimp-drawable-set-name fineDetailLayer "Fine (NrmInt 1-10)")
					(plug-in-gauss RUN-NONINTERACTIVE inImg fineDetailLayer (* UI_BlurPixels 0) (* UI_BlurPixels 0) FALSE)
				)
				(begin 
					(gimp-drawable-set-name fineDetailLayer "Fine (Gau 0px | NrmInt 1-10)")
				)
			)

			(if (= UI_Blending TRUE)
				(begin 
					(gimp-layer-set-mode fineDetailLayer LAYER-MODE-OVERLAY)
					(gimp-layer-set-opacity fineDetailLayer 90)
				)
			)

			(gimp-desaturate-full fineDetailLayer 1) ; Lightness 0, Luma 1, Avarage 2, Luminance 3, Value 4.


			; MEIUM NORMALMAP	
			(set! mediumDetailLayer (car (gimp-layer-copy inLayer FALSE)))
			(gimp-image-add-layer inImg mediumDetailLayer -2)
			

			(if (= UI_ApplyBlur TRUE)
				(begin 
					(gimp-drawable-set-name mediumDetailLayer "Medium (Nrm Int * 2X)")
					(plug-in-gauss RUN-NONINTERACTIVE inImg mediumDetailLayer (* UI_BlurPixels 4) (* UI_BlurPixels 4) FALSE)
				)
				(begin 
					(gimp-drawable-set-name mediumDetailLayer "Medium (Gau*2x | Nrm*2X)")
				)
			)

			(if (= UI_Blending TRUE)
				(begin 
					(gimp-layer-set-mode mediumDetailLayer LAYER-MODE-OVERLAY)
					(gimp-layer-set-opacity mediumDetailLayer 70)
				)
			)
			
			(gimp-desaturate-full mediumDetailLayer 1)
			(gimp-drawable-set-visible mediumDetailLayer FALSE)




			; LARGE NORMALMAP
			(set! largeDetailLayer (car (gimp-layer-copy inLayer FALSE)))
			(gimp-image-add-layer inImg largeDetailLayer -2)
			
			(if (= UI_ApplyBlur TRUE)
				(begin 
					(gimp-drawable-set-name largeDetailLayer "Large (NrmInt * 4X)")
					(plug-in-gauss RUN-NONINTERACTIVE inImg largeDetailLayer (* UI_BlurPixels 8) (* UI_BlurPixels 8) FALSE)
				)
				(begin 
					(gimp-drawable-set-name largeDetailLayer "Large (Gau*4x | Nrm*4X)")
				)
			)

			(if (= UI_Blending TRUE)
				(begin 
					(gimp-layer-set-mode largeDetailLayer LAYER-MODE-OVERLAY)
					(gimp-layer-set-opacity largeDetailLayer 50)
				)
			)
			
			(gimp-desaturate-full largeDetailLayer 1)
			(gimp-drawable-set-visible largeDetailLayer FALSE)

			(gimp-image-set-active-layer inImg fineDetailLayer)
			

		; : : : : : : : : : : : : : : : : : 

		
		; Keep Original Layer
		(if (= UI_KeepLayer FALSE)
			(gimp-image-remove-layer inImg inLayer)
		)


		; EDNS PROGRESS ::::::::::::::::::
		(gimp-progress-end)
		(gimp-image-undo-group-end inImg)
		(gimp-displays-flush)
		(gimp-context-pop)

	)

)

; UI OPTIONS
(script-fu-register "deos-MasterNormalMap"
    "Master Normal Map"
    "Create a normal map from multi layer details"
    "Deo"
    "Deo"
    "Feb, 2024"
    "RGB* GRAY*"
    SF-IMAGE        "image"                                                             0
    SF-DRAWABLE     "drawable"                                                          0
    SF-TOGGLE       "Keep Original Layer?"                                              TRUE
    SF-TOGGLE       "Apply 'Blur' to passes?"                                           TRUE
    SF-ADJUSTMENT   "Amount of Blur in Pixels"                                          '(1 0.1 32 1 2 1 0)	;Value, Min, Max, Step Min (Scroll), Step Max (Ctrl+Scroll), Digits, Silder Bool.
    SF-TOGGLE       "Apply 'Blending & Opacity' to passes?"                             TRUE
)

(script-fu-menu-register 	"deos-MasterNormalMap"
													"<Image>/Filters/Deo's Tools")