; ::::::::::::::::::
; UV PADDING TOOL
; ::::::::::::::::::


(define (deos-UVPadding inImg inLayer UI_Info UI_KeepLayer UI_ShowProces UI_Infinite UI_PerPixels)
	(let*
		( 
			; VARIABLES
			(editingLayer (car (gimp-layer-copy inLayer TRUE)))
			(alphaVal 0)
			(blurPadding 1)
			(blurFinalPass 5)
			(perPixels 0)
		)

		; START PROCESS :::::::::::::
		(gimp-context-push)
		(gimp-image-undo-group-start inImg)
		
		; Clean selection + duplicate on top
		(gimp-selection-none inImg)
		(gimp-image-add-layer inImg editingLayer (+ (car (gimp-image-get-layer-position inImg inLayer)) -1))
		(gimp-drawable-set-name editingLayer "UV Padding")

		; PADDING PROCESS :::::::::::::
		
		(if (= UI_Infinite TRUE) ; infinite

			(begin
				(while (< alphaVal 1)
					
					; Crete Border Selection
					(gimp-selection-layer-alpha editingLayer)
					(gimp-selection-border inImg 2)
					(gimp-selection-sharpen inImg)

					; Blur into Padding selection
					(if (equal? (car (gimp-selection-is-empty inImg)) FALSE)
						(plug-in-gauss RUN-NONINTERACTIVE inImg editingLayer blurPadding blurPadding FALSE)  
					)
					
					; Mid transparent px to opaque colors (Sharp Pixels)
					(gimp-levels editingLayer HISTOGRAM-ALPHA 0 128 1 0 255)	; InMin, InMax, Gamma(Med), OutMin, OutMax

					; Update in Editor the actions
					(if (= UI_ShowProces TRUE)
						(gimp-displays-flush)									
					)
					
					; Get the list of Opaque pixels only to get its information color 
					(set! alphaVal (list-ref (gimp-histogram editingLayer HISTOGRAM-ALPHA 255 255) 5)) ;ValMin, ValMax, inxed (A)
					(set! blurPadding (+ blurPadding 1))
				)
			)

			(begin 
				(while (< perPixels UI_PerPixels) ; per edges
					
					; Crete Border Selection
					(gimp-selection-layer-alpha editingLayer)
					(gimp-selection-border inImg 2)
					(gimp-selection-sharpen inImg)

					; Blur into Padding selection
					(if (equal? (car (gimp-selection-is-empty inImg)) FALSE)
						(plug-in-gauss RUN-NONINTERACTIVE inImg editingLayer blurPadding blurPadding FALSE)  
					)
					
					; Mid transparent px to opaque colors (Sharp Pixels)
					(gimp-levels editingLayer HISTOGRAM-ALPHA 0 128 1 0 255)	; InMin, InMax, Gamma(Med), OutMin, OutMax

					; Update in Editor the actions
					(if (= UI_ShowProces TRUE)
						(gimp-displays-flush)									
					)
					
					; Get the list of Opaque pixels only to get its information color 
					(set! alphaVal (list-ref (gimp-histogram editingLayer HISTOGRAM-ALPHA 255 255) 5)) ;ValMin, ValMax, inxed (A)
					(set! blurPadding (+ blurPadding 1))
					
					(set! perPixels (+ perPixels 1))
				)
			)

		)

		; BLUR INTO PADDING SECTION
		(gimp-selection-layer-alpha inLayer)
		(gimp-selection-invert inImg)
		(plug-in-gauss RUN-NONINTERACTIVE inImg editingLayer blurFinalPass blurFinalPass FALSE)
		(gimp-selection-none inImg)
		
		; : : : : : : : : : : : : : : : : : 
		

		; KEEP LAYER :::::::::::::::::::
		(if (= UI_KeepLayer FALSE)
			(gimp-image-remove-layer inImg inLayer)
		)

		; EDNS PROGRESS :::::::::::::::::::  
		(gimp-progress-end)
		(gimp-image-undo-group-end inImg)	
		(gimp-displays-flush)      
		(gimp-context-pop)

	)
)

(script-fu-register "deos-UVPadding"
		"UV Padding"
		"Add edge padding on transparent textures"
		"Deo"
		"Deo"
		"Feb, 2024"
		"RGB* GRAY*"
		SF-IMAGE      "image"      													0
		SF-DRAWABLE   "drawable"   													0
		SF-STRING			"Texture should be per UV Islands"	"[ With Alpha Channel ]"
		SF-TOGGLE			"Keep Original Layer?"								TRUE
		SF-TOGGLE			"Show Padding Process on each Step?"	TRUE
		SF-TOGGLE			"Apply Infinite UV Padding?"					FALSE
		SF-ADJUSTMENT	"Smooth Padding Per Pixels"				'(5 1 16 1 2 0 1)    ;Value, Min, Max, Step Min (Scroll), Step Max (Ctrl+Scroll), Digits, Silder Bool.
		
)

(script-fu-menu-register 	"deos-UVPadding"
													"<Image>/Filters/Deo's Tools")